# -*- coding: utf-8 -*-

""" Manage settings on a local node """

import nago.core
from nago.core import nago_access
import nago.protocols.httpserver
import unittest

@nago_access()
def start(debug=False):
    """ starts a nago agent (daemon) process """
    if debug:
        debug = True
    nago.protocols.httpserver.app.run(debug=debug)


@nago_access()
def stop(key, section='main'):
    """ stops the nago agent
    """
    pass





# -*- coding: utf-8 -*-

""" Run local nagios plugins """

import nago.core
from nago.core import nago_access
import nago.extensions.settings
import os.path
import pynag.Utils
import subprocess

@nago_access()
def get(search="unsigned"):
    """ List all available plugins"""
    plugins = []
    for i in os.walk('/usr/lib/nagios/plugins'):
        for f in i[2]:
            plugins.append(f)
    return plugins

@nago_access()
def run(plugin, *args, **kwargs):
    """ Run a specific plugin """
    plugin = '/usr/lib/nagios/plugins/' + plugin
    # todo: make sure plugin is inside plugindir
    command = [plugin] + list(args)
    p = subprocess.Popen(command, stdout=subprocess.PIPE,stderr=subprocess.PIPE,)
    stdout, stderr = p.communicate('through stdin to stdout')
    result = {}
    result['stdout'] = stdout
    result['stderr'] = stderr
    result['return_code'] = p.returncode
    return result



@nago_access()
def set(token_or_hostname, **kwargs):
    """ Change the attributes of a connected node """
    node = nago.core.get_node(security_token) or {}
    if not kwargs:
        return "No changes made"
    for k, v in kwargs.items():
        node[k] = v
    node.save()
    return "Saved %s changes" % len(kwargs)

@nago_access()
def connect(token_or_hostname=None):
    """ Connect to another node. By default connect to the masternode """
    if token_or_hostname is None:
        token_or_hostname = nago.extensions.settings.get('server')
    node = nago.core.get_node(token_or_hostname)
    if not node:
        try:
            address = socket.gethostbyname(token_or_hostname)
            node = nago.core.Node()
            node['host_name'] = token_or_hostname
            node['address'] = address
        except Exception:
            raise Exception("'%s' was not found in list of known hosts, and does not resolve to a valid address" % token_or_hostname)
    node.send_command('nodes', 'list')



# -*- coding: utf-8 -*-

""" Manage settings on a local node """

import ConfigParser
from nago.core import nago_access


@nago_access(name='set')
def edit_settings(section='main', **kwargs):
    """ Change a single option in local configuration """
    cfg_file = "/etc/nago/nago.ini"
    config = ConfigParser.ConfigParser()
    config.read(cfg_file)
    for k, v in kwargs.items():
        config.set(section=section, option=k, value=v)
    with open(cfg_file, 'w') as f:
        config.write(f)
    return "Done"


@nago_access(name='get')
def get(key, section='main'):
    """ Get a single option from """
    cfg_file = "/etc/nago/nago.ini"
    config = ConfigParser.ConfigParser()
    config.read(cfg_file)
    return config.get(section, key)



# -*- coding: utf-8 -*-

""" Management around connected nodes """

import nago.core
from nago.core import nago_access
import nago.extensions.settings

@nago_access(name='list')
def list_nodes(search="unsigned"):
    """ List all connected nodes
    """
    nodes = nago.core.get_nodes()
    if search == "all":
        return nodes
    elif search == 'unsigned':
        result = {}
        for token, node in nodes.items():
            if node.get('access') is None:
                result[token] = node
        return result
    else:
        result = {}
        for token, node in nodes.items():
            host_name = node.get('host_name')
            if search in (token, host_name):
                result[token] = node
        return result

@nago_access()
def sign(node=None):
    """ Sign a specific node to grant it access

     you can specify "all" to sign all nodes

     returns the nodes that were signed
    """
    if not node:
        raise Exception("Specify either 'all' your specify token/host_name of node to sign. ")
    if node == 'all':
        node = 'unsigned'
    nodes = list_nodes(search=node)
    result = {}
    for token, i in nodes:
        i['access'] = 'node'
        i.save()
        result[token] = i
    return result

@nago_access(name='set')
def set_attribute(token_or_hostname, **kwargs):
    """ Change the attributes of a connected node """
    node = nago.core.get_node(security_token) or {}
    if not kwargs:
        return "No changes made"
    for k, v in kwargs.items():
        node[k] = v
    node.save()
    return "Saved %s changes" % len(kwargs)

@nago_access()
def ping(token_or_hostname=None):
    """ Send an echo request to a nago host.

    Arguments:
        token_or_host_name  -- The remote node to ping
                            If node is not provided, simply return pong
                            You can use the special nodenames "server" or "master"
     """
    if not token_or_hostname:
        return "Pong!"
    node = nago.core.get_node(token_or_hostname)
    if not node and token_or_hostname in ('master', 'server'):
        token_or_hostname = nago.extensions.settings.get('server')
        node = nago.core.get_node(token_or_hostname)
    if not node:
        try:
            address = socket.gethostbyname(token_or_hostname)
            node = nago.core.Node()
            node['host_name'] = token_or_hostname
            node['address'] = address
        except Exception:
            raise Exception("'%s' was not found in list of known hosts, and does not resolve to a valid address" % token_or_hostname)
    return node.send_command('nodes', 'ping')




# -*- coding: utf-8 -*-

""" Get and post nagios checkresults between nago instances

This extension allows to get status data from a local nagios server.

Also pushing checkresults into a local nagios server, therefore updating nagios status.
"""
from pynag.Parsers import mk_livestatus, config
import time
import os
import os.path
import tempfile
from nago.core import nago_access
import nago.extensions.settings


@nago_access()
def get():
    """ Get all nagios status information from a local nagios instance
    """
    livestatus = mk_livestatus()
    hosts = livestatus.get_hosts()
    services = livestatus.get_services()
    result = {}
    result['hosts'] = hosts
    result['services'] = services
    return result

@nago_access()
def post(hosts=None, services=None, check_existance=True, create_services=True, create_hosts=False):
    """ Puts a list of hosts into local instance of nagios checkresults
    Arguments:
      hosts               -- list of dicts, like one obtained from get_checkresults
      services            -- list of dicts, like one obtained from get_checkresults
      check_existance     -- If True, check (and log) if objects already exist before posting
      create_services -- If True, autocreate non-existing services (where the host already exists)
      create_hosts    -- If True, autocreate non-existing hosts
    """

    nagios_config = config()
    nagios_config.parse_maincfg()
    check_result_path = nagios_config.get_cfg_value("check_result_path")


    fd, filename = tempfile.mkstemp(prefix='c', dir=check_result_path)
    if not hosts:
        hosts = []
    if not services:
        services = []

    if check_existance:
        checkresults_overhaul(hosts, services, create_services=create_services, create_hosts=create_hosts)
    checkresults = '### Active Check Result File Made by Nago ###\n'
    checkresults += 'file_time=%s' % (int(time.time()))
    checkresults = ''

    for host in hosts:
        checkresults += _format_checkresult(**host)
    for service in services:
        checkresults += _format_checkresult(**service)
    os.write(fd, checkresults)

    # Cleanup and make sure our file is readable by nagios
    os.close(fd)
    os.chmod(filename, 0644)

    # Create an ok file, so nagios knows it's ok to reap our changes
    file('%s.ok' % filename, 'w')

@nago_access()
def send(remote_host=None):
    """ Send local nagios data to a remote nago instance """
    my_data = get()
    if not remote_host:
        remote_host = nago.extensions.settings.get('server')
    remote_node = nago.core.get_node(remote_host)
    remote_node.send_command('checkresults', 'post', **my_data)
    return "checkresults sent to %s" % remote_host


def checkresults_overhaul(hosts, services, create_services, create_hosts):
    """ Iterates through hosts and services, and filters out those who do not exist in our local monitoring core

    If create_services or create_hosts are defined, then
    """


def _format_checkresult(**kwargs):
    """ Returns a string in a nagios "checkresults" compatible format """
    o = {}
    o['check_type'] = '1'
    o['check_options'] = '0'
    o['scheduled_check'] = '1'
    o['reschedule_check'] = '1'
    o['latency'] = '0.0'
    o['start_time'] = '%5f' % time.time()
    o['finish_time'] = '%5f' % time.time()
    o['early_timeout'] = '0'
    o['exited_ok'] = '1'
    o['long_plugin_output'] = ''
    o['performance_data'] = ''
    o.update(locals())
    o.update(kwargs)
    del o['kwargs']
    del o['o']

    template = _host_check_result
    # Escape all linebreaks if we have them
    for k, v in o.items():
        if isinstance(v, basestring) and '\n' in v:
            o[k] = v.replace('\n', '\\n')

    # Livestatus returns slightly different output than status.dat
    # Lets normalize everything to status.dat format
    if 'name' in o and not 'host_name' in o:
        o['host_name'] = o['name']
    if 'state' in o and not 'return_code' in o:
        o['return_code'] = o['state']
    if 'description' in o and not 'service_description' in o:
        o['service_description'] = o['description']
    if not o['performance_data'] and 'perf_data' in o:
        o['performance_data'] = o['perf_data']

    # If this is a service (as opposed to host) lets add service_description field in out putput
    if 'service_description' in o:
        template += "service_description={service_description}\n"
    if not o['performance_data'].endswith('\\n'):
        o['performance_data'] += '\\n'

    # Format the string and return
    return template.format(**o) + '\n'


# This is an example of what checkresult file looks like to nagios. This is used by
# _format_checkresult()
_host_check_result = """
host_name={host_name}
check_type={check_type}
check_options=0
scheduled_check=1
reschedule_check=1
latency=0.0
start_time={start_time}
finish_time={finish_time}
early_timeout=0
exited_ok=1
return_code={return_code}
output={plugin_output}{long_plugin_output} | {performance_data}
"""



from nago.core import nago_access, get_node
import nago.core
import inspect
import os
import json

""" Extensions module for Nago

All actual domain and check-specific logic of Nago should live as an extension.

Any method created here under @nago.core.nago_access decorator is automatically
made available to the message bus.

Take a look at facts.py for an example of a simple extension.
"""

__localaccess__ = object()  # If you are working locally on this machine, you can proof it with this
__loaded_extensions = {}    # Extensions are loaded here with the load() method


class NagoError(Exception):
    pass


class ExtensionError(NagoError):
    pass


@nago_access
def test():
    print __file__, __name__


def get_extension_names():
    """ Returns a list of the names of every loaded extension  """
    return get_extensions().keys()


def get_extensions():
    """ Returns all extensions in a dictionary  """
    return __loaded_extensions


def get_method_names(extension_name):
    """ List all remotely accessable methods of a specified extension """
    return sorted(get_methods(extension_name).keys())


def get_methods(extension_name):
    """ Return all methods in extension that have nago_access set """
    extension = get_extension(extension_name)
    methods = {}
    for name, i in inspect.getmembers(extension):
        if hasattr(i, 'nago_access'):
            api_name = i.nago_name
            methods[api_name] = i
    return methods


def get_method(extension_name, method_name):
    """ Return a specific python method """
    methods = get_methods(extension_name)
    return methods[method_name]


def get_extension(extension_name):
    return __loaded_extensions[extension_name]


def load(extension_name):
    try:
        extension = __import__(extension_name, globals(), locals(), [''])
        __loaded_extensions[extension_name] = extension
        if 'on_load' in dir(extension):
            extension.on_load()
        nago.core.log(level="info", message="API Extension loaded: %s" % extension_name)
    except Exception, e:
        nago.core.log("API Extension failed to load %s: %s" % (extension_name, e), level='error')


def call_method(token, extension_name, method_name, json_data=None, *args, **kwargs):
    """

    """
    method = get_method(extension_name, method_name)
    if not hasattr(method, 'nago_access'):
        raise NagoError("%s.%s is not remotely accessable" % (extension_name, method_name))

    # Check the security token, check if we have access to this method
    if token != __localaccess__:
        node = get_node(token)
        if node.get('access') != 'master' and node.get('access') != method.nago_access:
            result = {}
            result['status'] = 'error'
            result['message'] = "security token '%s' is authorized %s.%s" % (token, extension_name, method_name)
            return result

    if json_data:
        kwargs = kwargs.copy()
        data = json.loads(json_data)
        for k,v in data.items():
            kwargs[k] = v
    return method(*args, **kwargs)


if not __loaded_extensions:
    # Load all extensions in the same directory as the extensions package
    extension_dir = os.path.dirname(__file__)
    for dirpath, dirnames, filenames in os.walk(extension_dir):
        for f in filenames:
            if f.endswith('.py') and f[0].isalpha():
                module_name = f[:-3]
                load(module_name)



# setup.py ###
from distutils.core import setup
from setuptools import find_packages
import os


from nago import get_version

NAME = "nago"
VERSION = get_version()
SHORT_DESC = "General purpose agent, mostly used with nagios"
LONG_DESC = """ """

datafiles = []

datadirs = ['nago/protocols/httpserver/templates/', 'etc/nago']

for i in datadirs:
    for cur_dir, dirlist, filelist in os.walk(i):
        datafiles.append(("/" + cur_dir, filelist))
        print cur_dir, filelist, dirlist


if __name__ == "__main__":
    manpath = "share/man/man1/"
    etcpath = "/etc/%s" % NAME
    etcmodpath = "/etc/%s/modules" % NAME
    initpath = "/etc/init.d/"
    logpath = "/var/log/%s/" % NAME
    varpath = "/var/lib/%s/" % NAME
    rotpath = "/etc/logrotate.d"
    datarootdir = "/usr/share/%s" % NAME
    setup(
        name='%s' % NAME,
        version=VERSION,
        author='Pall Sigurdsson',
        description=SHORT_DESC,
        long_description=LONG_DESC,
        author_email='palli@opensource.is',
        url='http://github.com/opinkerfi/nago',
        license='GPL',
        scripts=['scripts/nago'],
        packages=find_packages(),
        data_files=datafiles,
    )




import time
import os
import os.path
import tempfile
import ConfigParser
import pwd

import json
import requests
import urllib

import platform
import nago
from functools import wraps
from pynag.Parsers import mk_livestatus, config

# List of log entries since program start. Format should be:
# [{'timestamp':x, 'level': x, 'message'}]
_log_entries = []


def log(message, level="info"):
    """ Add a new log entry to the nago log.

    Arguments:
        level - Arbritrary string, levels should be syslog style (debug,log,info,warning,error)
        message - Arbritary string, the message that is to be logged.
    """
    now = time.time()
    entry = {}
    entry['level'] = level
    entry['message'] = message
    entry['timestamp'] = now
    _log_entries.append(entry)
    if level not in ('debug', 'info'):
        print("{level}: {message}".format(**locals()))


def nago_access(access_type="master", name=None):
    """ Decorate other functions with this one to allow access

    Arguments:
        nago_access -- Type of access required to call this function
                       By default only master is allowed to make that call

        nago_name   -- What name this function will have to remote api
                       Default is the same as the name of the function being
                       decorated.
    """
    def real_decorator(func):
        func.nago_access = access_type
        func.nago_name = name or func.__name__
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)
        return wrapper
    return real_decorator


def get_nodes():
    """ Returns all nodes in a list of dicts format
    """
    cfg_file = "/etc/nago/nago.ini"
    config = ConfigParser.ConfigParser()
    config.read(cfg_file)
    result = {}
    for section in config.sections():
        if section in ['main']:
            continue
        token = section
        node = Node(token)
        for key, value in config.items(token):
            node[key] = value
        result[token] = node
    return result


def get_node(token_or_hostname):
    for name, node in get_nodes().items():
        if token_or_hostname in (name, node.get('host_name'), node.get('address'), node.get('uri')):
            return node
    return None


def has_access(token):
    """ Returns true if specified token exists and is marked as "allowed" """
    node = get_node(token) or {}
    if node.get('access'):
        return True
    return False


import os
import string
import ConfigParser


def generate_token():
    """ Generate a new random security token.

     >>> len(generate_token()) == 50
     True

     Returns:
       string
    """
    length = 50
    stringset = string.ascii_letters + string.digits
    token = ''.join([stringset[i % len(stringset)] for i in [ord(x) for x in os.urandom(length)]])
    return token


class Node(object):
    """ Represents one specific node (another nago client) """
    def __init__(self, token=None, **kwargs):
        if not token:
            token = generate_token()
        self.data = kwargs.copy()
        self.token = token
        self._original_token = token

    def __getitem__(self, item):
        return self.data[item]

    def get(self, attribute, default=None):
        return self.data.get(attribute, default)

    def __setitem__(self, item, value):
        self.data[item] = value

    def save(self):
        """ Save this node (and all its attributes) to config """
        cfg_file = "/etc/nago/nago.ini"
        config = ConfigParser.ConfigParser()
        config.read(cfg_file)
        result = {}
        token = self.data.pop("token", self.token)
        if token != self._original_token:
            config.remove_section(self._original_token)
            config.add_section(token)

        if token in config.sections():
            for key, value in self.data.items():
                config.set(token, key, value)
            for key, value in config.items(token):
                if key not in self.data:
                    config.set(token, key, None)
        with open(cfg_file, 'w') as f:
            return config.write(f)

    def __str__(self):
        return self.data.__str__()

    def __repr__(self):
        return self.data.__repr__()

    def send_command(self, extension_name, method_name, **kwargs):
        uri = self.get('uri')
        address = self.get('address')
        port = self.get('port') or 5000
        port = str(port)
        arguments = kwargs.copy()
        arguments['token'] = arguments.pop('token', self.token)

        if not uri and not address:
            raise Exception("We need either a remote address or uri to connect to node")
        elif not uri:
            uri = "http://{address}:{port}".format(**locals())

        uri += "/api/{extension_name}/{method_name}?".format(**locals())
        querystring = urllib.urlencode(arguments.items())

        uri += querystring
        log("Connecting to {uri}".format(**locals()), level="debug")
        try:
            content = requests.get(uri).content
            log(message="Successfully connected to %s" % uri, level="debug")
            results = json.loads(content)
            return results
        except Exception, e:
            results = {}
            results['error'] = "Failed to connect to %s" % uri
            results['error_type'] = type(e)
            results['details'] = e
            results['status'] = 'error'
            return results


def get_my_info():
    """ Return general information about this node
    """
    result = {}
    result['host_name'] = platform.node()
    result['real_host_name'] = platform.node()
    result['dist'] = platform.dist()
    result['nago_version'] = nago.get_version()
    return result


# -*- coding: utf-8 -*-
""" Extension for the facter command

This extension allows getting facts (via 'facter' command) about this host,

posting facts is also supported, which means the agent will store facts about a remote server in memory.

"""

from nago.core import nago_access
from nago.core import get_nodes
import json
facts = {}
from pynag.Utils import runCommand

@nago_access()
def get():
    """ Get local facts about this machine.

    Returns:
        json-compatible dict with all facts of this host
    """
    result = runCommand('facter --json', raise_error_on_fail=True)
    json_facts = result[1]
    facts = json.loads(json_facts)
    return facts

@nago_access()
def post(host_token, **kwargs):
    """ Store facts about a remote host in memory.

    Facts will be stored in nago.extensions.facts with the remote token as a key

    """
    facts[host_token] = kwargs
    return "thanks!"


@nago_access()
def get_all():
    """ Get all facts about all nodes """
    return facts

@nago_access()
def send(remote_host=None):
    """ Send my facts to a remote host

    if remote_host is provided, data will be sent to that host. Otherwise it will be sent to master.
    """
    my_facts = get()
    remote_node = None  # Here we will store the Node we connect to

    for token, i in get_nodes().items():
        if token == remote_host:
            remote_node = i
            break
        elif remote_host is None and i.get('access') == 'master':
            remote_node = i
            break
    if not remote_node:
        raise Exception("Remote host with token='%s' not found" % remote_host)
    result = remote_node.send_command('facts', 'post', host_token=remote_node.token, **my_facts)
    return "sent %s facts to remote node '%s'" % (len(my_facts), remote_node.token)




if __name__ == '__main__':
    my_facts = get_facts()
    post_facts("123", my_facts)

    print facts






#!/usr/bin/env python
# -*- coding: utf-8 -*-
""" A http server written in flask. Allows two or more nago instances to talk with each other

"""
from flask import Flask, request, session, g, redirect, url_for, abort, render_template, flash, jsonify
from functools import wraps
import nago.core
import nago.extensions

# this is a workaround for rhel6 systems where python-jinja has multiple versions
# installed at the same time
# https://bugzilla.redhat.com/show_bug.cgi?id=867105
__requires__ = ['jinja2 >= 2.4']
import pkg_resources


app = Flask(__name__)
app.secret_key = 'bla'


def login_required(func, permission=None):
    """ decorate with this function in order to require a valid token for any view

     If no token is present you will be sent to a login page
    """
    @wraps(func)
    def decorated_function(*args, **kwargs):
        if not check_token():
            return login()
        elif not nago.core.has_access(session.get('token')):
            return http403()
        return func(*args, **kwargs)
    return decorated_function





def check_token():
    if 'token' in request.args :
        session['token'] = request.args.get('token')
    elif request.method == 'POST' and 'token' in request.form:
        session['token'] = request.form.get('token')

    if session.get('token'):
        return True
    else:
        return False


@app.route('/403', methods=['GET', 'POST'])
def http403():
    token = session.get('token')
    node = nago.core.get_node(token) or {}
    host_name = node.get('host_name') or ''
    return render_template('403.html', **locals())

@app.route('/', methods=['GET', 'POST'])
@login_required
def index():
    token = session.get('token')
    node = nago.core.get_node(token)

    my_info = nago.core.get_my_info()

    extension_dict = nago.extensions.get_extensions()
    extensions = {}
    for k, v in extension_dict.items():
        extensions[k] = {}
        extensions[k]['description'] = v.__doc__
        extensions[k]['shortdesc'] = str(v.__doc__).splitlines()[0]
        extensions[k]['methods'] = {}
        for name in nago.extensions.get_method_names(k):
            method = nago.extensions.get_method(k, name)
            extensions[k]['methods'][name] = {}
            extensions[k]['methods'][name]['description'] = method.__doc__
    return render_template('index.html', **locals())


@app.route('/login', methods=['GET', 'POST'])
def login():
    return render_template('login.html')



@app.route('/esxtensions/<extension_name>/', methods=['GET', 'POST'])
@login_required
def extension(extension_name):
    token = session.get('token')
    node = nago.core.get_node(token) or {}
    host_name = node.get('host_name') or ''
    return render_template('index.html', **locals())


@app.route('/extensions/', methods=['GET'])
@login_required
def list_extensions():
    token = session.get('token')
    node = nago.core.get_node(token) or {}
    host_name = node.get('host_name') or ''
    extensions = nago.extensions.get_extensions()
    result = {}
    for k, v in extensions.items():
        result[k] = {}
        result[k]['description'] = v.__doc__
        result[k]['methods'] = {}
        for name in nago.extensions.get_method_names(k):
            method = nago.extensions.get_method(k, name)
            result[k]['methods'][name] = {}
            result[k]['methods'][name]['description'] = method.__doc__

    return jsonify(**result)



@app.route('/api/<extension_name>/<method_name>/', methods=['GET', 'POST'])
@login_required
def call_method(extension_name, method_name):
    kwargs = {}
    for k, v in request.args.items():
        kwargs[k] = v
    kwargs.pop('token', None)
    kwargs.pop('json_data', None)
    token = session['token']
    result = {}
    result['result'] = nago.extensions.call_method(token=token, extension_name=extension_name, method_name=method_name, **kwargs)
    return jsonify(**result)


if __name__ == '__main__':
    app.run(debug=True)


# -*- coding: utf-8 -*-

""" Collect and transmit information about local node """

import nago.core
from nago.core import nago_access
import nago.protocols.httpserver
import nago.extensions.settings
import unittest

node_data = {}

@nago_access()
def get_info(node_name=None, key=None):
    """ Get information about this node """
    if node_name is None:
        node_name = nago.core.get_my_info()['host_name']
    data = node_data.get(node_name, {})
    if not key:
        return data
    else:
        return data.get(key)



@nago_access()
def post(node_name, key, data):
    """ Give the server information about this node

    Arguments:
        node -- node_name or token for the node this data belongs to
        key  -- identifiable key, that you use later to retrieve that piece of data
        data -- the data you need to store

    """
    node = node_data.get(node_name, {})
    node[key] = data
    node_data[node_name] = node
    return "thanks!"


def on_load():
    my_info = nago.core.get_my_info()
    post(node_name=my_info.get('host_name'), key="my_info", data=my_info)


class TestInfo(unittest.TestCase):
    """ Unit Tests for the info extension """
    def test_get_info(self):
        # Call on_load() to load local info to memory
        on_load()

        # Get our own basic info, double check nago version number
        results = get_info()
        version = results.get('my_info', {}).get('nago_version')
        self.assertEqual(nago.get_version(), version)

        # Try post some changes and see if we can fetch them back
        host_name = results.get('my_info', {}).get('host_name')
        key = 'test'
        data = 'test data'
        tmp = post(host_name, key, data)
        self.assertEqual('thanks!', tmp)

        # get the same data back, make sure it matches
        posted_data = get_info(host_name, key)
        self.assertEqual(data, posted_data)